; Azimut Project
; Copyright  2008 Philippe Coulombe. All rights reserved.

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; AzmtStr Macro
;; Creates ANSI/UNICODE null-char terminated string.
;;
;; This macro must be used in the .DATA section.
;;
;; You can't use SIZEOF or LENGTHOF with the strings created with this macro.
;;
;; Without any special characters you can create a 245 characters length string.
;; This length is reduced by the number of special characters used and the label name.
;; This limit comes from MASM. Error A2041 : string or text literal too long...
;; "A string or text literal, or a macro function return value, exceeded the limit of 255 characters"
;;
;;
;; StrLbl (required) : String label.
;; StrTyp (required) : String type (BYTE or WORD or TCHAR).
;; StrTxt (optional) : String text. Some characters can't be specified in the string text because they
;;                     are reserved by the macro pre-processor. For these special cases you can use the
;;                     slash followed by a replacement character (see below).
;; BufLen (optional) : Buffer length including the string itself (character count and not the byte count
;;                     of the buffer). This length is not affected by the MASM limit described above.
;;
;;
;; Special characters :  // = slash
;;                       /{ = parenthese open
;;                       /} = parenthese close
;;                       /[ = chevron open
;;                       /] = chevron close
;;                       /a = apostrophe or single quote
;;                       /b = backspace
;;                       /e = exclamation
;;                       /f = form feed
;;                       /l = line feed
;;                       /n = new line (cr+lf)
;;                       /p = percent
;;                       /q = double quote
;;                       /r = carriage return
;;                       /s = space (only needed for a single character string)
;;                       /t = tab
;;                       /v = vertical tab
;;                       /0 = null character
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; .DATA
;;
;; AzmtStr szMacroName ,TCHAR, <AzmtStr Macro/0>
;; AzmtStr szCopyright ,TCHAR, <Copyright  Philippe Coulombe. All rights reserved./0>
;;
;; .CODE
;;
;; invoke MessageBox, NULL, addr szCopyright, addr szMacroName, MB_OK
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

AzmtStr MACRO StrLbl:REQ, StrTyp:REQ, StrTxt, BufLen:=<0>

    LOCAL flg
    LOCAL cnt
    LOCAL bln
    LOCAL lbl

    ;
    ; Default values of local variables
    ;

    flg = 0                             ; flag set when a special character is found
    cnt = 0                             ; holds the string length
    bln = BufLen                        ; holds the buffer length
    lbl textequ <&StrLbl>               ; holds the label name

    ;
    ; Compute the string length
    ;

    ifnb <&StrTxt>
        forc chr, <&StrTxt>
            if flg ne 0
                if '&chr' eq 'n'
                    cnt = cnt + 2
                else
                    cnt = cnt + 1
                endif
                flg = 0
            elseif '&chr' eq '/'
                flg = 1
            else
                cnt = cnt + 1
            endif
        endm
    endif

    ;
    ; If flg = 1 after the loop above it means that the string text have a
    ; trailing slash and special characters won't be parsed correctly
    ;

    if flg eq 1
        .err <AzmtStr Macro Error: A special character is needed after a slash>
    endif

    ;
    ; Write the string
    ;

    ifnb <&StrTxt>
        forc chr, <&StrTxt>
            if flg ne 0
                if '&chr' eq '/'        ; // = slash
                    lbl StrTyp '/'
                elseif '&chr' eq '{'    ; /{ = parenthese open
                    lbl StrTyp '('
                elseif '&chr' eq '}'    ; /} = parenthese close
                    lbl StrTyp ')'
                elseif '&chr' eq '['    ; /[ = chevron open
                    lbl StrTyp '<'
                elseif '&chr' eq ']'    ; /] = chevron close
                    lbl StrTyp '>'
                elseif '&chr' eq 'a'    ; /a = apostrophe or single quote
                    lbl StrTyp "'"
                elseif '&chr' eq 'b'    ; /b = backspace
                    lbl StrTyp 8
                elseif '&chr' eq 'e'    ; /e = exclamation
                    lbl StrTyp '!'
                elseif '&chr' eq 'f'    ; /f = form feed
                    lbl StrTyp 12
                elseif '&chr' eq 'l'    ; /l = line feed
                    lbl StrTyp 10
                elseif '&chr' eq 'n'    ; /n = new line (cr+lf)
                    lbl StrTyp 13,10
                elseif '&chr' eq 'p'    ; /p = percent
                    lbl StrTyp '%'
                elseif '&chr' eq 'q'    ; /q = double quote
                    lbl StrTyp '"'
                elseif '&chr' eq 'r'    ; /r = carriage return
                    lbl StrTyp 13
                elseif '&chr' eq 's'    ; /s = space (only needed for a single character string)
                    lbl StrTyp 32
                elseif '&chr' eq 't'    ; /t = tab
                    lbl StrTyp 9
                elseif '&chr' eq 'v'    ; /v = vertical tab
                    lbl StrTyp 11
                elseif '&chr' eq '0'    ; /0 = null character
                    lbl StrTyp 0
                else
                    .err <AzmtStr Macro Error: An invalid special character has been specified>
                endif
                flg = 0
                lbl textequ <>          ; lbl is only needed once for the label definition
            elseif '&chr' eq '/'
                flg = 1
            else
                lbl StrTyp '&chr'
                lbl textequ <>          ; lbl is only needed once for the label definition
            endif
        endm
    endif

    ;
    ; Appends null-chars if a buffer length has been specified
    ;

    if bln gt cnt
        StrTyp bln - cnt dup (0)
    elseif bln ne 0
        .err <AzmtStr Macro Error: The buffer length is not large enough to hold the string>
    endif

ENDM

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

NULL                        EQU 0
FALSE                       EQU 0

ANYSIZE_ARRAY               EQU 1

DUPLICATE_CLOSE_SOURCE      EQU 00000001h
DUPLICATE_SAME_ACCESS       EQU 00000002h

LMEM_FIXED                  EQU 0000h

MAX_PATH                    EQU 260

ObjectNameInformation       EQU 1

PROCESS_DUP_HANDLE          EQU 0040h
PROCESS_QUERY_INFORMATION   EQU 0400h
PROCESS_VM_READ             EQU 0010h

SE_PRIVILEGE_ENABLED        EQU 00000002h

STATUS_SUCCESS              EQU 00000000h

SystemHandleInformation     EQU 16

TOKEN_ADJUST_PRIVILEGES     EQU 0020h
TOKEN_QUERY                 EQU 0008h

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

HANDLEINFO STRUCT
    Pid                     DWORD ?
    ObjectType              WORD ?
    HandleValue             WORD ?
    ObjectPointer           DWORD ?
    AccessMask              DWORD ?
HANDLEINFO ENDS

LUID STRUCT
    LowPart                 DWORD ?
    HighPart                DWORD ?
LUID ENDS

LUID_AND_ATTRIBUTES STRUCT
    Luid                    LUID <>
    Attributes              DWORD ?
LUID_AND_ATTRIBUTES ENDS

SYSTEMHANDLEINFO STRUCT
    nHandleEntries          DWORD ?
    HandleInfo              HANDLEINFO 1 DUP (<>)
SYSTEMHANDLEINFO ENDS

TOKEN_PRIVILEGES STRUCT
    PrivilegeCount          DWORD ?
    Privileges              LUID_AND_ATTRIBUTES ANYSIZE_ARRAY DUP (<>)
TOKEN_PRIVILEGES ENDS

UNICODE_STRING STRUCT
    _Length                 WORD ?
    MaximumLength           WORD ?
    Buffer                  DWORD ?
UNICODE_STRING ENDS

OBJECT_NAME_INFORMATION STRUCT
    _Name                   UNICODE_STRING <>
    NameBuffer              WORD MAX_PATH*2 DUP (?)
OBJECT_NAME_INFORMATION ENDS

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
